/* client.hh: XMMS2 client header.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#ifndef GX2OSD_CLIENT_HH
#define GX2OSD_CLIENT_HH

#include <glibmm/ustring.h>
#include <gdkmm/pixbuf.h>

#include <xmmsclient/xmmsclient++.h>

#include "boost/noncopyable.hpp"

#include "window_holder.hh"
#include "cache.hh"

namespace gx2osd {

    class client : private boost::noncopyable {

        public:

            client();
            ~client();

        private:

            bool entry_changed( const unsigned int& id );
            bool song_changed( const unsigned int& id );
            bool error_handler( const std::string& error,
                                const Xmms::PropDict& info );
            bool info_get( const Xmms::PropDict& info );
            bool cover_get( const Xmms::bin& cover,
                            const std::string& cover_hash,
                            const Xmms::PropDict& info );

            Glib::RefPtr< Gdk::Pixbuf >
            scale_pixbuf( Glib::RefPtr< Gdk::Pixbuf > const& pixbuf ) const;

            void dc_handler();

            Xmms::Client client_;
            window_holder& win_;
            unsigned int curid_;
            ecli::cache< std::string, Glib::RefPtr< Gdk::Pixbuf > > cover_cache_;

    };

}

#endif // GX2OSD_CLIENT_HH
