/* main_window.hh: OSD Window header.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#ifndef GX2OSD_MAIN_WINDOW_HH
#define GX2OSD_MAIN_WINDOW_HH

#include <gtkmm/window.h>
#include <gtkmm/label.h>
#include <gtkmm/fixed.h>
#include <gtkmm/image.h>

#include <glibmm/ustring.h>

#include <xmmsclient/xmmsclient++.h>

#include "skin.hh"
#include "scroller.hh"

#include "boost/noncopyable.hpp"
#include "boost/shared_ptr.hpp"
#include "boost/tuple/tuple.hpp"

namespace gx2osd {

    class main_window : public Gtk::Window, private boost::noncopyable {

        public:

            main_window( Glib::RefPtr< Gdk::Screen > screen );
            virtual ~main_window();

            void set_text( const Xmms::PropDict& info,
                           unsigned int extra_width = 0 );
            void set_text_and_show( const Xmms::PropDict& info );

            void set_cover( const Glib::RefPtr< Gdk::Pixbuf >& cover,
                            const Xmms::PropDict& );

            bool set_status_no_show( const Xmms::Playback::Status& status );

        private:
            typedef boost::shared_ptr< eclidgets::scroller > scroller_ptr;
            typedef boost::tuple< scroller_ptr, Glib::ustring,
                                  gx2osd::label > label_tuple;
            
            void load_skin();
            void set_label_stuff( eclidgets::scroller& gtklabel,
                                  const gx2osd::label& label,
                                  Glib::ustring& markup );
            virtual bool on_button_press_event( GdkEventButton* event );

            std::list< label_tuple > labels_;

            Gtk::Fixed fixed_pane_;
            const Glib::ustring span_end_;

            Gtk::Image background_;
            Gtk::Image cover_;
            Gtk::Image status_;

            Glib::RefPtr< Gdk::Screen > screen_;

            const boost::shared_ptr< gx2osd::skin >& skin_;

    };

    main_window& get_window();

}

#endif // GX2OSD_MAIN_WINDOW_HH
