#ifndef PLAYLIST_LANGUAGE_HH
#define PLAYLIST_LANGUAGE_HH

#include <iostream>
#include <string>
#include <xmmsclient/xmmsclient++/dict.h>
#include <boost/shared_ptr.hpp>
#include <boost/variant.hpp>
#include <boost/function.hpp>
#include <boost/optional.hpp>
#include <vector>

namespace playlist_language
{

namespace id
{
enum PLL_ID
{
    BLOCK = 1,
    FUNCTION,
    STRING,
    ID,
    TAG,
    ARG,
    NOT,
    INT,
    DOUBLE,
    FACTOR,
    TERM,
    CALC,
    CMP
};
}

class evaluator
{

public:
    typedef boost::variant< int, double, std::string > variable_type;
    typedef std::vector< variable_type > arguments;
    typedef boost::optional< variable_type > return_t;
    typedef boost::function< return_t( arguments const&,
                                       std::ostream& ) > func_t;

    evaluator(){}
    virtual ~evaluator() {}

    virtual void
    set_info( Xmms::Dict const& info ) = 0;

    virtual void
    run( std::ostream& os = std::cout ) = 0;

    virtual void
    run( Xmms::Dict const& info, std::ostream& os = std::cout ) = 0;

    virtual bool
    add_function( std::string const& name, func_t func ) = 0;

};

typedef boost::shared_ptr< evaluator > evaluator_ptr;

evaluator_ptr
parse_file( std::string const& file );

evaluator_ptr
parse_data( std::string const& data );

}

#endif // PLAYLIST_LANGUAGE_HH
