#include "key_combination.hh"

#include <stdexcept>
#include <iostream>
#include <string>
#include <list>

#include <glib/gtypes.h>
#include <gdk/gdktypes.h>
#include <gdk/gdkkeysyms.h>
#include <gdk/gdkkeys.h>

#include <boost/foreach.hpp>
#include <boost/algorithm/string/case_conv.hpp>
#include <boost/algorithm/string/classification.hpp>
#include <boost/algorithm/string/split.hpp>
#include <boost/range.hpp>

ecli::key_combination::key_combination( std::string const& combination )
    : modifiers_( 0 ), key_( 0 )
{
    parse_combination( combination );
}

ecli::key_combination::~key_combination()
{
}

guint
ecli::key_combination::get_modifiers() const
{
    return modifiers_;
}

guint
ecli::key_combination::get_key() const
{
    return key_;
}

void
ecli::key_combination::parse_combination( std::string const& combination )
{
    using boost::algorithm::token_compress_on;
    using boost::algorithm::to_lower_copy;
    using boost::algorithm::is_any_of;
    using boost::algorithm::split;
    using boost::iterator_range;
    typedef iterator_range< std::string::iterator > char_range;
    typedef std::list< char_range > item_list;

    std::string temp( to_lower_copy( combination ) );
    item_list items;
    split( items, temp, is_any_of( " +" ), token_compress_on );
    BOOST_FOREACH( char_range i, items )
    {
        if( i == items.back() ) {
            std::string temp2( i.begin(), i.end() );
            key_ = gdk_keyval_from_name( temp2.c_str() );
            if( key_ == GDK_VoidSymbol ) {
                throw std::invalid_argument( "Not a valid key name: " + temp2 );
            }
        }
        else {
            for( conversion_table* c = conversion_table_;
                 c->value != GDK_VoidSymbol; ++c )
            {
                if( c->name == i ) {
                    modifiers_ |= c->value;
                    break;
                }
            }
        }
    }
}

ecli::key_combination::conversion_table
ecli::key_combination::conversion_table_[] =
{
    { "shift", GDK_SHIFT_MASK },
    { "ctrl", GDK_CONTROL_MASK },
    { "alt", GDK_MOD1_MASK },
    { "meta", GDK_META_MASK },
    { "", GDK_VoidSymbol }
};
