/* skin.hh: skin format header.
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */

#ifndef GX2OSD_SKIN_HH
#define GX2OSD_SKIN_HH

#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMElement.hpp>

#include "boost/noncopyable.hpp"
#include "boost/shared_ptr.hpp"
#include "boost/filesystem/path.hpp"

#include <string>
#include <map>

#include "image.hh"
#include "label.hh"

namespace fs = boost::filesystem;

namespace gx2osd {

    class skin : private boost::noncopyable {

        public:
            typedef std::map< std::string, gx2osd::label > label_map;

            skin( const std::string& name );
            ~skin();

            bool has_cover() const;

            image& background();
            image& no_cover_background();
            image& no_cover_image();
            image& paused_image();
            image& stopped_image();
            image& playing_image();

            const position& get_status_position() const;

            const label_map& get_labels() const;

        private:

            typedef boost::shared_ptr< XMLCh > xmlch;

            const std::string name_;
            fs::path skin_path_;
            
            bool has_cover_;

            image background_;
            image no_cover_background_;
            image no_cover_image_;
            image paused_image_;
            image stopped_image_;
            image playing_image_;

            std::map< std::string, gx2osd::label > labels_;

            position status_position_;

            void load_image( image& i, xmlch tag_name,
                             xercesc::DOMDocument*& doc, xmlch attribute );

            void load_labels( xercesc::DOMDocument*& doc );

            void parse_position( xercesc::DOMElement*& element,
                                 unsigned int& x, unsigned int& y );

            std::pair< xercesc::DOMElement*, xercesc::DOMNodeList* >
            find_element( xmlch tag_name, xercesc::DOMDocument*& doc );

            void handle_label( xercesc::DOMElement* element );

    };

}

#endif // GX2OSD_SKIN_HH
