/* window_holder.hh: Proxy class to control multiple windows at once
 *
 * Copyright (c) 2006 Jonne Lehtinen <jms@modeemi.fi>
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the Do What The F*ck You Want To
 *  Public License, Version 2, as published by Sam Hocevar. See
 *  COPYING file for more details.
 */
#ifndef GX2OSD_WINDOW_HOLDER_HH
#define GX2OSD_WINDOW_HOLDER_HH

#include <xmmsclient/xmmsclient++/playback.h>
#include <xmmsclient/xmmsclient++/dict.h>
#include <boost/noncopyable.hpp>
#include <boost/scoped_ptr.hpp>
#include <glibmm/ustring.h>
#include <glibmm/refptr.h>
#include <gdkmm/pixbuf.h>
#include <sigc++/connection.h>
#include <list>

namespace gx2osd {

    class main_window;
    class window_holder : private boost::noncopyable {

        public:
            ~window_holder();
            static window_holder& get_instance();

            void add_window( gx2osd::main_window* window );
            void show_all();
            bool set_status_no_show( const Xmms::Playback::Status& s );
            bool set_status( const Xmms::Playback::Status& s );
            void set_text_and_show( const Xmms::PropDict& info );
            void set_cover( const Glib::RefPtr< Gdk::Pixbuf >& cover,
                            const Xmms::PropDict& info );

        private:
            window_holder();
            bool hider();
            static boost::scoped_ptr< window_holder > instance_;
            std::list< gx2osd::main_window* > windows_;
            sigc::connection conn_;

    };

    window_holder& get_window_holder();

}

#endif
